#!/usr/bin/env python
from __future__ import generators
# import findrox; findrox.version(1,9,12)
# it uses a patched version of ROX-Lib2 in rox/
import rox
from rox import g, tasks
import os
import gc
import gobject

#global minsize ### a Identifier for miniatur icons
#global minsize



class EventBuffers:
	def __init__(self):
		self.pixbuf = None
		self.pixbuf_active = None
		self.saturation = 5
	def set_saturation(self, sat):
		self.saturation = sat
	def set_image(self):
		self.iconset = g.IconSet(self.pixbuf)
		if self.pixbuf_active: del self.pixbuf_active; self.pixbuf_active = None; gc.collect()
		self.pixbuf_active = self.pixbuf.copy()
		self.pixbuf_active.saturate_and_pixelate(self.pixbuf_active, self.saturation, g.FALSE)
		self.iconsrc = g.IconSource()
		self.iconsrc.set_pixbuf(self.pixbuf_active)
		self.iconsrc.set_state_wildcarded(g.FALSE)
		self.iconsrc.set_state(g.STATE_PRELIGHT)
		self.iconset.add_source(self.iconsrc)
	def get_from_file(self, parent, Filename):
		if os.path.isfile(Filename):
			if self.pixbuf: del self.pixbuf; self.pixbuf = None; gc.collect()
    			try:
				self.pixbuf = g.gdk.pixbuf_new_from_file(Filename)
				self.set_image()
			except (gobject.GError):
				#print gobject.GError
				self.set_from_stock(parent, g.STOCK_MISSING_IMAGE)
		else:
			self.set_from_stock(parent, g.STOCK_MISSING_IMAGE)
	def set_from_stock(self, parent, Stock):
		if self.pixbuf: del self.pixbuf; self.pixbuf = None; gc.collect()
		self.pixbuf = parent.render_icon(Stock, g.ICON_SIZE_SMALL_TOOLBAR, "")
		self.set_image()
	

class EventImage(g.EventBox):
	def __init__(self, Stock = None, press_event = None, tooltip = None):
		g.EventBox.__init__(self)
		self.image = g.Image()
		self.add(self.image)
		self.EventBuf = EventBuffers()
		self.set_events(g.gdk.ENTER_NOTIFY_MASK | g.gdk.LEAVE_NOTIFY_MASK)
		if press_event <> None:
			self.connect('button_press_event', self.button_pressed)
			self.connect('button_press_event', press_event)
			self.connect('button_release_event', self.button_released)
		self.connect("enter-notify-event", self.enter_widget)
		self.connect("leave-notify-event", self.leave_widget)
		if Stock: self.set_from_stock(Stock)
		else: self.set_from_stock(g.STOCK_MISSING_IMAGE)
    		self.image.show()
		self.show()
		if tooltip: self.set_tip(tooltip)
	def button_pressed(self, widget, event):
		widget.set_state (g.STATE_SELECTED)
		tasks.Task(self.blink())
		#print "button pressed"
	def button_released(self, widget, event):
		widget.set_state (g.STATE_NORMAL)
		#print "button released"
	def blink(self):
		#import time
		for i in range(0, 6):
			self.set_state(g.STATE_PRELIGHT)
			yield tasks.TimeoutBlocker(0.02)
			#time.sleep(0.05)
			self.set_state(g.STATE_NORMAL)
			yield tasks.TimeoutBlocker(0.02)
			#time.sleep(0.05)
			
	def set_small(self):
		import RETicker
		self.image.set_from_icon_set(self.EventBuf.iconset,  RETicker.minsize)
	def get_from_file(self,Filename):
		self.EventBuf.get_from_file(self, Filename)
		self.image.set_from_icon_set(self.EventBuf.iconset, g.ICON_SIZE_SMALL_TOOLBAR)
	def set_from_stock(self, Stock):
		self.EventBuf.set_from_stock(self, Stock)
		self.image.set_from_icon_set(self.EventBuf.iconset, g.ICON_SIZE_SMALL_TOOLBAR)
	def enter_widget(self, widget, event):
		widget.set_state (g.STATE_PRELIGHT)
	def leave_widget(self, widget, event):
		widget.set_state (g.STATE_NORMAL)
	def set_tip(self, Text):
		if not(Text): return
		self.tooltip = g.Tooltips(); self.tooltip.enable()
		self.tooltip.set_tip(self, Text)
	def rem_tip(self):
		self.tooltip.disable()

import re
def unxml(string):
	""" removes tags and entities from a given string and returns
	new string """
	lt=re.compile('&lt;')
	gt=re.compile('&gt;')
	amp=re.compile('&amp;')
	auml=re.compile('&auml;|&#228;')
	Auml=re.compile('&Auml;')
	ouml=re.compile('&ouml;|&#246;')
	Ouml=re.compile('&Ouml;')
	uuml=re.compile('&uuml;|&#252;')
	Uuml=re.compile('&Uuml;')
	szlig=re.compile('&szlig;|&#223;')
	quot=re.compile('&quot;')
	apos=re.compile('&apos;')
	#tag=re.compile('<.*?>')
	(string,null)=amp.subn("&",string,0)
	(string,null)=auml.subn(u'\xE4',string,0)
	(string,null)=Auml.subn(u'\xC4',string,0)
	(string,null)=ouml.subn(u'\xF6',string,0)
	(string,null)=Ouml.subn(u'\xD5',string,0)
	(string,null)=Uuml.subn(u'\xDC',string,0)
	(string,null)=szlig.subn(u'\xDF',string,0)
	(string,null)=uuml.subn(u'\xFC',string,0)
	(string,null)=quot.subn('"',string,0)
	(string,null)=apos.subn("'",string,0)
	(string,null)=lt.subn("<",string,0)
	(string,null)=gt.subn(">",string,0)
	#(string,null)=tag.subn("",string,0)
	return string

import time
def conv_iso_time(date):
	if date[-5] == "+" or date[-5] == "-": date = date[:-5]
	if date[-6] == "+" or date[-6] == "-": date = date[:-6]
	return time.mktime(time.strptime(date, "%Y-%m-%dT%H:%M:%S"))

def getdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	return c_name

def getdata_wo_ret(channel,name): ### get data without return
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	pos = c_name.find('\n')
	while pos <> -1:
		if pos <> 0: tmp = c_name[:pos] + " "
		else: tmp = ""
		tmp += c_name[pos+1:]
		c_name = tmp
		pos = c_name.find('\n')
	return c_name

def getintdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	try: return int(c_name)
	except: return 0
def getfloatdata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	try: return float(c_name)
	except: return 0

def gettimedata(channel,name):
	c_name_s=channel.getElementsByTagName(name)
	c_name = ""
	if c_name_s:
		for t in c_name_s[0].childNodes:
			if t: c_name+=t.data
	if c_name == "": return 0
	else: return float(conv_iso_time(c_name))
def quit(self, event):
	print "event set_from_stock"
	tool.set_from_stock(g.STOCK_OPEN)
	#tool.get_from_file("/home/rene/Choices/RETicker_0.4/icon_cache/slashdot.org_favicon.ico")

def get_prelight_colors(Options):
	back_obj = g.gdk.color_parse(Options.back_color.value)
	text_obj = g.gdk.color_parse(Options.text_color.value)
	if Options.transparent.int_value == 1:
		if text_obj.red + text_obj.green + text_obj.blue > 65535/2:
			text_obj = change_color(text_obj, 0.3)
		else: text_obj = change_color(text_obj, -0.3)
	if text_obj.red + text_obj.green + text_obj.blue \
		> back_obj.red + back_obj.green + back_obj.blue:
		# Text heller als Hintergrund
		text_obj = change_color(text_obj, 0.3)
		back_obj = change_color(back_obj, -0.3)
	else:
		text_obj = change_color(text_obj, -0.3)
		back_obj = change_color(back_obj, 0.3)
	return text_obj, back_obj

def set_color_part(color_part, value):
	tmp = int(color_part * (1 + value))
	if tmp > 65535: return 65535
	if tmp < 0: return 0 # this shouldnt happen ;)
	return tmp

def change_color(obj, value):	
	obj.red = set_color_part(obj.red, value)
	obj.green = set_color_part(obj.green, value)
	obj.blue = set_color_part(obj.blue, value)
	obj.pixel = 0
	return obj


def back_color_prelight(Options):
	text_obj, back_obj = get_prelight_colors(Options)
	return back_obj

def text_color_prelight(Options):
	text_obj, back_obj = get_prelight_colors(Options)
	return text_obj

def tmp_name(cache, url):
	pos = url.find("http://")
	if pos <> -1: url = url[pos+7:]
	while 1:
		pos = url.find("/")
		if pos <> -1:
			url = url[:pos] + "_" + url[pos+1:]
		else: break
	return cache + "/" + url

#main = g.Window()
#main.show()
#tool = EventImage(g.STOCK_CANCEL,quit)
#main.add(tool)
#tool.get_from_file("/home/rene/Choices/RETicker_0.4/icon_cache/slashdot.org_favicon.ico")
#g.mainloop()
#time.sleep(3)
#tool.set_from_stock(g.STOCK_OPEN)


def test(hallo): print hallo

class WaitContainer:
	def __init__(self, time_to_wait, function_to_call):
		self.time_to_wait = time_to_wait
		self.function_to_call = function_to_call
		self.value = None
		self.timeout = None
	def set_time(self, time):
		self.time_to_wait = time
	def set_value(self, value):
		self.value = value
		#print "set_value"
		if self.timeout: g.timeout_remove(self.timeout)
		self.timeout = g.timeout_add(long(self.time_to_wait * 1000), self.fire)
	def fire(self):
		#print "fired"
		self.fire_task = None
		self.function_to_call(self.value)
		self.value = None

#test = WaitContainer(0.5, test)
#test.set_value("aha")
#time.sleep(3)
#test.set_value("2")
#test.set_value("afw")
#test.set_value("fff")

#g.mainloop()
